/* 
   Linux launcher

   Copyright (c) 2011 Fabrice Bellard

   Redistribution or commercial use is prohibited without the author's permission.
*/
"use strict";

load("term.js");
load("cpux86-ta.js");

var term, pc, boot_start_time;

function term_start()
{
    term = new Term(80, 30, term_handler);

    term.open();
}

/* send chars to the serial port */
function term_handler(str)
{
    pc.serial.send_chars(str);
}

function clipboard_set(val)
{
    var el;
    el = document.getElementById("text_clipboard");
    el.value = val;
}

function clipboard_get()
{
    var el;
    el = document.getElementById("text_clipboard");
    return el.value;
}

function clear_clipboard()
{
    var el;
    el = document.getElementById("text_clipboard");
    el.value = "";
}

/* just used to display the boot time in the VM */
function get_boot_time()
{
    return (+new Date()) - boot_start_time;
}

function start()
{
    var start_addr, initrd_size, params, cmdline_addr;
    
    params = new Object();

    /* serial output chars */
    params.serial_write = term.write.bind(term);

    /* memory size (in bytes) */
    params.mem_size = 16 * 1024 * 1024;

    /* clipboard I/O */
    params.clipboard_get = clipboard_get;
    params.clipboard_set = clipboard_set;

    params.get_boot_time = get_boot_time;

    pc = new PCEmulator(params);

    pc.load_binary("vmlinux26.bin", 0x00100000);

    initrd_size = pc.load_binary("root.bin", 0x00400000);

    start_addr = 0x10000;
    pc.load_binary("linuxstart.bin", start_addr);

    /* set the Linux kernel command line */
    /* Note: we don't use initramfs because it is not possible to
       disable gzip decompression in this case, which would be too
       slow. */
    cmdline_addr = 0xf800;
    pc.cpu.write_string(cmdline_addr, "console=ttyS0 root=/dev/ram0 rw init=/sbin/init notsc=1");

    pc.cpu.eip = start_addr;
    pc.cpu.regs[0] = params.mem_size; /* eax */
    pc.cpu.regs[3] = initrd_size; /* ebx */
    pc.cpu.regs[1] = cmdline_addr; /* ecx */

    boot_start_time = (+new Date());

    pc.start();
}

term_start();

//XXX -------- SHELL hacks ---------

var timeouts = [];
function setTimeout(f, t) {
    timeouts.push(f);
}

var lastY = 0;

function __output(y, s) {
    s = s.replace(/&nbsp;/g, " ");
    if (lastY === y)
        putstr("\r");
    else
        putstr("\n");
    putstr(s.trim());
    lastY = y;
}

CPU_X86.prototype.load_binary = function(Xe, fa) {
    var Ze = snarf(Xe, "binary");

    var cd = Ze.byteLength;
    var af = new Uint8Array(Ze, 0, cd);

    for (var i = 0; i < cd; i++) {
        this.st8_phys(fa + i, af[i]);
    }
    return cd;
}


start();

while (timeouts.length > 0) {
    var f = timeouts.shift();
    f();
}

